@@ -1,6 +1,8 @@
 %%
 %% %CopyrightBegin%
 %%
+%% SPDX-License-Identifier: Apache-2.0
+%%
 %% Copyright Ericsson AB 1999-2025. All Rights Reserved.
 %%
 %% Licensed under the Apache License, Version 2.0 (the "License");
@@ -26,16 +28,11 @@
          bad_arglist/1,
 	 equality/1,ordering/1,
 	 fun_to_port/1,t_phash/1,t_phash2/1,md5/1,
-	 refc/1,refc_ets/1,refc_dist/1,
 	 const_propagation/1,t_arity/1,t_is_function2/1,
-	 t_fun_info/1,t_fun_info_mfa/1,t_fun_to_list/1,
-         spurious_badfun/1]).
+	 t_fun_info/1,t_fun_info_mfa/1,t_fun_to_list/1]).
 
 -export([nothing/0]).
 
-%% Callback for a process that uses this module as an error_handler module.
--export([undefined_lambda/3]).
-
 -include_lib("common_test/include/ct.hrl").
 
 suite() ->
@@ -47,10 +44,9 @@ all() ->
     [bad_apply, bad_fun_call, badarity, ext_badarity,
      bad_arglist,
      equality, ordering, fun_to_port, t_phash,
-     t_phash2, md5, refc, refc_ets, refc_dist,
+     t_phash2, md5,
      const_propagation, t_arity, t_is_function2, t_fun_info,
-     t_fun_info_mfa,t_fun_to_list,
-     spurious_badfun].
+     t_fun_info_mfa,t_fun_to_list].
 
 init_per_testcase(_TestCase, Config) ->
     Config.
@@ -347,9 +343,7 @@ ordering(Config) when is_list(Config) ->
 
     %% Create a port and ref.
 
-    Path = proplists:get_value(priv_dir, Config),
-    AFile = filename:join(Path, "vanilla_file"),
-    P = open_port(AFile, [out]),
+    P = hd(erlang:ports()),
     R = make_ref(),
 
     %% Compare funs with ports and refs.
@@ -409,20 +403,23 @@ make_fun(X, Y) ->
 
 %% Try sending funs to ports (should fail).
 fun_to_port(Config) when is_list(Config) ->
-    fun_to_port(Config, xxx),
-    fun_to_port(Config, fun() -> 42 end),
-    fun_to_port(Config, [fun() -> 43 end]),
-    fun_to_port(Config, [1,fun() -> 44 end]),
-    fun_to_port(Config, [0,1|fun() -> 45 end]),
+    Port = open_port({spawn_executable, os:find_executable("erl")},
+                     [{args, ["-noshell", "-eval", "timer:sleep(2000)",
+                              "-run", "erlang", "halt"]},
+                      use_stdio]),
+    fun_to_port(Port, xxx),
+    fun_to_port(Port, fun() -> 42 end),
+    fun_to_port(Port, [fun() -> 43 end]),
+    fun_to_port(Port, [1,fun() -> 44 end]),
+    fun_to_port(Port, [0,1|fun() -> 45 end]),
     B64K = build_io_list(65536),
-    fun_to_port(Config, [B64K,fun() -> 45 end]),
-    fun_to_port(Config, [B64K|fun() -> 45 end]),
+    fun_to_port(Port, [B64K,fun() -> 45 end]),
+    fun_to_port(Port, [B64K|fun() -> 45 end]),
+    unlink(Port),
+    exit(Port, kill),
     ok.
 
-fun_to_port(Config, IoList) ->
-    Path = proplists:get_value(priv_dir, Config),
-    AFile = filename:join(Path, "vanilla_file"),
-    Port = open_port(AFile, [out]),
+fun_to_port(Port, IoList) ->
     case catch port_command(Port, IoList) of
 	{'EXIT',{badarg,_}} -> ok;
 	Other -> ct:fail({unexpected_retval,Other})
@@ -512,165 +509,18 @@ md5(Config) when is_list(Config) ->
 bad_md5(Bad) ->
     {'EXIT',{badarg,_}} = (catch erlang:md5(Bad)).
 
-refc(Config) when is_list(Config) ->
-    F1 = fun_factory(2),
-    {refc,2} = erlang:fun_info(F1, refc),
-    F2 = fun_factory(42),
-    {refc,3} = erlang:fun_info(F1, refc),
-
-    process_flag(trap_exit, true),
-    Pid = spawn_link(fun() -> {refc,4} = erlang:fun_info(F1, refc) end),
-    receive
-	{'EXIT',Pid,normal} -> ok;
-	Other -> ct:fail({unexpected,Other})
-    end,
-    process_flag(trap_exit, false),
-    %% Wait to make sure that the process has terminated completely.
-    receive after 1 -> ok end,
-    {refc,3} = erlang:fun_info(F1, refc),
-
-    %% Garbage collect. Only the F2 fun will be left.
-    7 = F1(5),
-    true = erlang:garbage_collect(),
-    40 = F2(-2),
-    {refc,2} = erlang:fun_info(F2, refc),
-    ok.
-
-fun_factory(Const) ->
-    fun(X) -> X + Const end.
-
-refc_ets(Config) when is_list(Config) ->
-    F = fun(X) -> X + 33 end,
-    {refc,2} = erlang:fun_info(F, refc),
-
-    refc_ets_set(F, [set]),
-    refc_ets_set(F, [ordered_set]),
-    refc_ets_bag(F, [bag]),
-    refc_ets_bag(F, [duplicate_bag]),
-    ok.
-
-refc_ets_set(F1, Options) ->
-    io:format("~p", [Options]),
-    Tab = ets:new(kalle, Options),
-    true = ets:insert(Tab, {a_key,F1}),
-    3 = fun_refc(F1),
-    [{a_key,F3}] = ets:lookup(Tab, a_key),
-    4 = fun_refc(F1),
-    true = ets:insert(Tab, {a_key,not_a_fun}),
-    3 = fun_refc(F1),
-    true = ets:insert(Tab, {another_key,F1}),
-    4 = fun_refc(F1),
-    true = ets:delete(Tab),
-    3 = fun_refc(F1),
-    10 = F3(-23),
-    true = erlang:garbage_collect(),
-    2 = fun_refc(F1),
-    ok.
-
-refc_ets_bag(F1, Options) ->
-    io:format("~p", [Options]),
-    Tab = ets:new(kalle, Options),
-    true = ets:insert(Tab, {a_key,F1}),
-    3 = fun_refc(F1),
-    [{a_key,F3}] = ets:lookup(Tab, a_key),
-    4 = fun_refc(F1),
-    true = ets:insert(Tab, {a_key,not_a_fun}),
-    4 = fun_refc(F1),
-    true = ets:insert(Tab, {another_key,F1}),
-    5 = fun_refc(F1),
-    true = ets:delete(Tab),
-    3 = fun_refc(F1),
-    10 = F3(-23),
-    true = erlang:garbage_collect(),
-    2 = fun_refc(F1),
-    ok.
-
-refc_dist(Config) when is_list(Config) ->
-    {ok, Peer, Node} = ?CT_PEER(),
-    process_flag(trap_exit, true),
-    Pid = spawn_link(Node, fun() -> receive
-                                        Fun when is_function(Fun) ->
-                                            3 = fun_refc(Fun),
-                                            exit({normal,Fun}) end
-                           end),
-    F = fun() -> 42 end,
-    2 = fun_refc(F),
-    Pid ! F,
-    F2 = receive
-	     {'EXIT',Pid,{normal,Fun}} -> Fun;
-	     Other -> ct:fail({unexpected,Other})
-	 end,
-    %% dist.c:net_mess2 have a reference to Fun for a while since
-    %% Fun is passed in an exit signal. Wait until it is gone.
-    wait_until(fun () -> 4 =/= fun_refc(F2) end),
-    3 = fun_refc(F2),
-    true = erlang:garbage_collect(),
-    2 = fun_refc(F),
-    refc_dist_send(Node, F),
-    peer:stop(Peer).
-
-refc_dist_send(Node, F) ->
-    Pid = spawn_link(Node, fun() -> receive
-                                        {To,Fun} when is_function(Fun) ->
-                                            wait_until(fun () ->
-                                                               3 =:= fun_refc(Fun)
-                                                       end),
-                                            To ! Fun
-                                    end
-                           end),
-    2 = fun_refc(F),
-    Pid ! {self(),F},
-    F2 = receive
-	     Fun when is_function(Fun) -> Fun;
-	     Other -> ct:fail({unexpected,Other})
-	 end,
-    receive {'EXIT',Pid,normal} -> ok end,
-    %% No reference from dist.c:net_mess2 since Fun is passed
-    %% in an ordinary message.
-    3 = fun_refc(F),
-    3 = fun_refc(F2),
-    refc_dist_reg_send(Node, F).
-
-refc_dist_reg_send(Node, F) ->
-    true = erlang:garbage_collect(),
-    2 = fun_refc(F),
-    Ref = make_ref(),
-    Me = self(),
-    Pid = spawn_link(Node, fun() ->
-                                   true = register(my_fun_tester, self()),
-                                   Me ! Ref,
-                                   receive
-                                       {Me,Fun} when is_function(Fun) ->
-                                           3 = fun_refc(Fun),
-                                           Me ! Fun
-                                   end
-                           end),
-    erlang:yield(),
-    2 = fun_refc(F),
-    receive Ref -> ok end,
-    {my_fun_tester,Node} ! {self(),F},
-    F2 = receive
-	     Fun when is_function(Fun) -> Fun;
-	     Other -> ct:fail({unexpected,Other})
-	 end,
-    receive {'EXIT',Pid,normal} -> ok end,
-
-    3 = fun_refc(F),
-    3 = fun_refc(F2),
-    ok.
-    
 fun_refc(F) ->
     {refc,Count} = erlang:fun_info(F, refc),
     Count.
 
 const_propagation(Config) when is_list(Config) ->
     Fun1 = fun wait_until/1,
-    2 = fun_refc(Fun1),
+    1 = fun_refc(Fun1),
     Fun2 = Fun1,
     my_cmp({Fun1,Fun2}),
 
     Fun3 = fun() -> ok end,
-    2 = fun_refc(Fun3),
+    1 = fun_refc(Fun3),
     Fun4 = Fun3,
     my_cmp({Fun3,Fun4}),
     ok.
@@ -703,20 +553,36 @@ t_arity(Config) when is_list(Config) ->
 t_is_function2(Config) when is_list(Config) ->
     false = is_function(id({a,b}), 0),
     false = is_function(id({a,b}), 234343434333433433),
-    true = is_function(fun() -> ok end, 0),
-    true = is_function(fun(_) -> ok end, 1),
-    false = is_function(fun(_) -> ok end, 0),
+    true = is_function(id(fun() -> ok end), 0),
+    true = is_function(id(fun(_) -> ok end), 1),
+    false = is_function(id(fun(_) -> ok end), 0),
 
-    true = is_function(fun erlang:abs/1, 1),
-    true = is_function(fun erlang:abs/99, 99),
-    false = is_function(fun erlang:abs/1, 0),
-    false = is_function(fun erlang:abs/99, 0),
+    true = is_function(id(fun erlang:abs/1), 1),
+    true = is_function(id(fun erlang:abs/99), 99),
+    false = is_function(id(fun erlang:abs/1), 0),
+    false = is_function(id(fun erlang:abs/99), 0),
 
     false = is_function(id(self()), 0),
     false = is_function(id({a,b,c}), 0),
     false = is_function(id({a}), 0),
     false = is_function(id([a,b,c]), 0),
 
+    %% Larger arities.
+    F16 = id(fun f/16),
+    F255 = id(fun f/255),
+
+    false = is_function(id(self()), 16),
+    true = is_function(F16, 16),
+    ok = id(if is_function(F16, 16) -> ok; true -> error end),
+    false = is_function(F255, 16),
+    error = id(if is_function(F255, 16) -> ok; true -> error end),
+
+    false = is_function(id(self()), 255),
+    true = is_function(F255, 255),
+    false = is_function(F16, 255),
+    error = id(if is_function(F16, 255) -> ok; true -> error end),
+    ok = id(if is_function(F255, 255) -> ok; true -> error end),
+
     %% Bad arity argument.
     bad_arity(a),
     bad_arity(-1),
@@ -728,7 +594,7 @@ t_is_function2(Config) when is_list(Config) ->
     bad_arity(self()),
 
     %% Bad arity argument in guard test.
-    Fun = fun erlang:abs/1,
+    Fun = id(fun erlang:abs/1),
     ok = if
              is_function(Fun, -1) -> error;
              is_function(Fun, 256) -> error;
@@ -738,6 +604,44 @@ t_is_function2(Config) when is_list(Config) ->
          end,
     ok.
 
+f(_A1, _A2, _A3, _A4, _A5, _A6, _A7, _A8,
+  _A9, _A10, _A11, _A12, _A13, _A14, _A15, _A16) ->
+    ok.
+
+f(_A1, _A2, _A3, _A4, _A5, _A6, _A7, _A8,
+  _A9, _A10, _A11, _A12, _A13, _A14, _A15, _A16,
+  _A17, _A18, _A19, _A20, _A21, _A22, _A23, _A24,
+  _A25, _A26, _A27, _A28, _A29, _A30, _A31, _A32,
+  _A33, _A34, _A35, _A36, _A37, _A38, _A39, _A40,
+  _A41, _A42, _A43, _A44, _A45, _A46, _A47, _A48,
+  _A49, _A50, _A51, _A52, _A53, _A54, _A55, _A56,
+  _A57, _A58, _A59, _A60, _A61, _A62, _A63, _A64,
+  _A65, _A66, _A67, _A68, _A69, _A70, _A71, _A72,
+  _A73, _A74, _A75, _A76, _A77, _A78, _A79, _A80,
+  _A81, _A82, _A83, _A84, _A85, _A86, _A87, _A88,
+  _A89, _A90, _A91, _A92, _A93, _A94, _A95, _A96,
+  _A97, _A98, _A99, _A100, _A101, _A102, _A103, _A104,
+  _A105, _A106, _A107, _A108, _A109, _A110, _A111, _A112,
+  _A113, _A114, _A115, _A116, _A117, _A118, _A119, _A120,
+  _A121, _A122, _A123, _A124, _A125, _A126, _A127, _A128,
+  _A129, _A130, _A131, _A132, _A133, _A134, _A135, _A136,
+  _A137, _A138, _A139, _A140, _A141, _A142, _A143, _A144,
+  _A145, _A146, _A147, _A148, _A149, _A150, _A151, _A152,
+  _A153, _A154, _A155, _A156, _A157, _A158, _A159, _A160,
+  _A161, _A162, _A163, _A164, _A165, _A166, _A167, _A168,
+  _A169, _A170, _A171, _A172, _A173, _A174, _A175, _A176,
+  _A177, _A178, _A179, _A180, _A181, _A182, _A183, _A184,
+  _A185, _A186, _A187, _A188, _A189, _A190, _A191, _A192,
+  _A193, _A194, _A195, _A196, _A197, _A198, _A199, _A200,
+  _A201, _A202, _A203, _A204, _A205, _A206, _A207, _A208,
+  _A209, _A210, _A211, _A212, _A213, _A214, _A215, _A216,
+  _A217, _A218, _A219, _A220, _A221, _A222, _A223, _A224,
+  _A225, _A226, _A227, _A228, _A229, _A230, _A231, _A232,
+  _A233, _A234, _A235, _A236, _A237, _A238, _A239, _A240,
+  _A241, _A242, _A243, _A244, _A245, _A246, _A247, _A248,
+  _A249, _A250, _A251, _A252, _A253, _A254, _A255) ->
+    ok.
+
 bad_arity(A) ->
     {'EXIT',_} = (catch is_function(fun() -> ok end, A)),
     {'EXIT',_} = (catch is_function(no_fun, A)),
@@ -839,79 +743,11 @@ verify_not_undef(Fun, Tag) ->
 	{Tag,_} -> ok
     end.
 
-%% Test for a race condition that occurred when multiple processes
-%% attempted to a call a fun whose defining module was not loaded.
-spurious_badfun(Config) ->
-    Mod = ?FUNCTION_NAME,
-    Dir = proplists:get_value(priv_dir, Config),
-    File = filename:join(Dir, atom_to_list(Mod) ++ ".erl"),
-
-    Code = <<"
-    -module(spurious_badfun).
-    -export([factory/0]).
-    factory() ->
-        fun() -> ok end.
-    ">>,
-
-    ok = file:write_file(File, Code),
-
-    {ok,Mod,Bin} = compile:file(File, [binary]),
-    {module,Mod} = erlang:load_module(Mod, Bin),
-    Fun = Mod:factory(),
-
-    do_spurious_badfun(1000, Mod, Bin, Fun).
-
-do_spurious_badfun(0, _Mod, _Bin, _Fun) ->
-    ok;
-do_spurious_badfun(N, Mod, Bin, Fun) ->
-    _ = catch erlang:purge_module(Mod),
-    _ = erlang:delete_module(Mod),
-    _ = catch erlang:purge_module(Mod),
-
-    Prepared = erlang:prepare_loading(Mod, Bin),
-
-    {Pid,Ref} = spawn_monitor(fun() -> call_fun(Fun) end),
-
-    ok = erlang:finish_loading([Prepared]),
-
-    receive
-        {'DOWN',Ref,process,Pid,Result} ->
-            normal = Result,
-            do_spurious_badfun(N-1, Mod, Bin, Fun)
-    end.
-
-call_fun(Fun) ->
-    %% Set up the current module as the error_handler for the current
-    %% process.
-    process_flag(error_handler, ?MODULE),
-
-    %% With the JIT, the following call would sometimes fail with a
-    %% `badfun` exeception. The reason is that the native code and the
-    %% C function beam_jit_handle_unloaded_fun() handling an unloaded
-    %% fun would use different code indexes. The native code would
-    %% "think" that the module for the fun was not loaded, while
-    %% beam_jit_handle_unloaded_fun() function would "think" that the
-    %% module was loaded and raise a badfun exception.
-    Fun().
-
-%% This is the error_handler callback for the process that is calling
-%% the fun.
-undefined_lambda(_Module, Fun, Args) ->
-    %% If the parent process has finished loading the module, the
-    %% following apply/2 call will succeed. Otherwise, this function
-    %% will be called again.
-    apply(Fun, Args).
-
-%%%
-%%% Common utilities.
-%%%
-
 id(X) ->
     X.
 
 spawn_call(Node, AFun) ->
-    Parent = self(),
-    Init = erlang:whereis(init),
+    Self = self(),
     Pid = spawn_link(Node,
 		     fun() ->
 			     receive
@@ -922,10 +758,7 @@ spawn_call(Node, AFun) ->
 						_ -> lists:seq(0, Arity-1)
 					    end,
 				     Res = apply(Fun, Args),
-                     case erlang:fun_info(Fun, pid) of
-                        {pid,Init} -> Parent ! {result,Res};
-                        {pid,Creator} -> Creator ! {result,Res}
-                     end
+                                     Self ! {result,Res}
 			     end
 		     end),
     Pid ! {AFun,AFun,AFun},
