@@ -1,7 +1,9 @@
 %%
 %% %CopyrightBegin%
 %%
-%% Copyright Ericsson AB 2003-2024. All Rights Reserved.
+%% SPDX-License-Identifier: Apache-2.0
+%%
+%% Copyright Ericsson AB 2003-2025. All Rights Reserved.
 %%
 %% Licensed under the Apache License, Version 2.0 (the "License");
 %% you may not use this file except in compliance with the License.
@@ -1447,11 +1449,10 @@ the new timetrap.
 """.
 -doc(#{since => <<"OTP R14B">>}).
 -spec timetrap(Time) -> infinity | pid()
-      when Time :: {hours, Hours} | {minutes, Mins} | {seconds, Secs} | Millisecs | infinity | Func,
+      when Time :: {hours, Hours} | {minutes, Mins} | {seconds, Secs} | timeout() | Func,
            Hours :: integer(),
            Mins :: integer(),
            Secs :: integer(),
-           Millisecs :: integer(),
            Func :: {M, F, A} | function(),
            M :: atom(),
            F :: atom(),
@@ -1469,7 +1470,7 @@ Note the `Time` is not the scaled result.
 """.
 -doc(#{since => <<"OTP R15B">>}).
 -spec get_timetrap_info() -> {Time, {Scaling,ScaleVal}}
-      when Time :: integer() | infinity,
+      when Time :: timeout(),
            Scaling :: boolean(),
            ScaleVal :: integer().
 get_timetrap_info() ->
@@ -1484,11 +1485,11 @@ up the time automatically if `scale_timetraps` is set to `true` (default is
 """.
 -doc(#{since => <<"OTP R14B">>}).
 -spec sleep(Time) -> ok
-      when Time :: {hours, Hours} | {minutes, Mins} | {seconds, Secs} | Millisecs | infinity,
+      when Time :: {hours, Hours} | {minutes, Mins} | {seconds, Secs} | Millisecs,
            Hours :: integer(),
            Mins :: integer(),
            Secs :: integer(),
-           Millisecs :: integer() | float().
+           Millisecs :: timeout() | float().
 sleep({hours,Hs}) ->
     sleep(trunc(Hs * 1000 * 60 * 60));
 sleep({minutes,Ms}) ->
