@@ -1,8 +1,10 @@
 %%
 %% %CopyrightBegin%
-%% 
+%%
+%% SPDX-License-Identifier: Apache-2.0
+%%
 %% Copyright Ericsson AB 2002-2025. All Rights Reserved.
-%% 
+%%
 %% Licensed under the Apache License, Version 2.0 (the "License");
 %% you may not use this file except in compliance with the License.
 %% You may obtain a copy of the License at
@@ -14,12 +16,13 @@
 %% WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 %% See the License for the specific language governing permissions and
 %% limitations under the License.
-%% 
+%%
 %% %CopyrightEnd%
 %%
 %%
 
 -module(httpc_handler).
+-moduledoc false.
 
 -behaviour(gen_server).
 
@@ -221,6 +224,8 @@ init([Parent, Request, Options, ProfileName]) ->
 
     %% Do not let initial tcp-connection block the manager-process
     proc_lib:init_ack(Parent, self()),
+    {Host, Port} = Request#request.address,
+    proc_lib:set_label({Request#request.scheme, erlang:iolist_to_binary(Host), Port}),
     handle_verbose(Options#options.verbose),
     ProxyOptions = handle_proxy_options(Request#request.scheme, Options),
     Address = handle_proxy(Request#request.address, ProxyOptions),
@@ -1316,9 +1321,9 @@ handle_server_closing(State = #state{headers = Headers}) ->
     end.
 
 answer_request(#request{id = RequestId, from = From, request_options = Options} = Request, Msg,
-	       #state{session      = Session, 
-		      timers       = Timers, 
-		      profile_name = ProfileName} = State) ->
+               #state{session      = Session,
+                      timers       = Timers,
+                  profile_name = ProfileName} = State) ->
     Answer = format_answer(Msg, Options),
     httpc_response:send(From, Answer),
     RequestTimers = Timers#timers.request_timers,
@@ -1713,20 +1718,6 @@ format_address({[$[|T], Port}) ->
 format_address(HostPort) ->
     HostPort.
 
-clobber_and_retry(#state{session = #session{id = Id,
-                                            type = Type},
-                         profile_name = ProfileName,
-                         pipeline = Pipeline,
-                         keep_alive = KeepAlive} = State) ->
-    %% Clobber session
-    (catch httpc_manager:delete_session(Id, ProfileName)),
-    case Type of
-        pipeline ->
-            maybe_retry_queue(Pipeline, State);
-        _ ->
-            maybe_retry_queue(KeepAlive, State)
-    end.
-    
 format_answer(Res, Options) ->
     FullResult = proplists:get_value(full_result, Options, true),
     Sync = proplists:get_value(sync, Options, true),
@@ -1761,4 +1752,18 @@ do_format_answer({Ref, {StatusLine, _, BinBody}}, false, Sync) ->
             {Ref, {Status, BinBody}}
     end;
 do_format_answer({Ref, {error, _Reason} = Error}, _, _) ->
-    {Ref, Error}.
\ No newline at end of file
+    {Ref, Error}.
+
+clobber_and_retry(#state{session = #session{id = Id,
+                                            type = Type},
+                         profile_name = ProfileName,
+                         pipeline = Pipeline,
+                         keep_alive = KeepAlive} = State) ->
+    %% Clobber session
+    (catch httpc_manager:delete_session(Id, ProfileName)),
+    case Type of
+        pipeline ->
+            maybe_retry_queue(Pipeline, State);
+        _ ->
+            maybe_retry_queue(KeepAlive, State)
+    end.
